import pytest
import click
from click.testing import CliRunner
from os import path
from sib.installer import Installer

class TestInstaller(object):

    @pytest.fixture(autouse=True)
    def setup(self, tmp_path):
        self.installer = Installer("sibtest", path.join(tmp_path,'sibtest'), True)

    def test_create_project(self):
        result = self.installer.create_project()
        assert result == True



import venv
import subprocess
from os import mkdir, path, environ
from pkg_resources import resource_filename


class Installer(object):

    """
    This class initialize a new SIB project by installing the required components and configuring the project
    """

    def __init__(self, project_name, project_folder=None, venv=False, modules=[]):

        """Init"""

        self.project_name = project_name

        if project_folder:
            self.project_folder = path.abspath(project_folder)
        else:
            self.project_folder = path.abspath(self.project_name)

        if venv:
            self.venv_path = path.join(self.project_folder, 'env')
        else:
            self.venv_path = None

        # add required modules if needed
        self.modules = modules

    def get_env(self):

        """Return environment"""

        env = environ.copy()

        # override the PATH with virtualenv property if any
        if self.venv_path:
            env['PATH'] = path.join(self.venv_path, 'bin') + ":" + env['PATH']

        return env

    def get_template(self):

        """Return the path of django project template from package resouces"""

        return resource_filename(__name__, 'template')

    def create_project(self):

        """Create the SIB project"""

        try:
            # create project folder
            mkdir(self.project_folder)

            # create virtualenv
            if self.venv_path:
                venv.EnvBuilder(with_pip=True).create(self.venv_path)
                # upgrade pip
                cmd = ['pip', 'install', '-U', 'pip']
                subprocess.run(cmd, env=self.get_env())

            # install modules including required ones
            cmd = ['pip', 'install', '-U']
            cmd.extend(['django-guardian', 'djangoldp'])
            cmd.append('requests')
            cmd.extend(self.modules)
            subprocess.run(cmd, env=self.get_env()).check_returncode()

            # init django
            cmd = [
                'django-admin',
                'startproject',
                '--template=' + self.get_template(),
                self.project_name,
                self.project_folder
            ]
            subprocess.run(cmd, env=self.get_env()).check_returncode()

            # configure LDP modules overring package.py file
            try:
                with open(path.join(self.project_folder, self.project_name, 'packages.py'), 'w') as f:
                    f.write("DJANGOLDP_PACKAGES=[\n'djangoldp',\n'")
                    f.write("',\n'".join(self.modules))
                    f.write("']")
            except IOError:
                print('Error locating packages.py')
                pass

            # make migrations
            cmd = ['python', 'manage.py', 'makemigrations']
            subprocess.run(cmd, env=self.get_env(), cwd=self.project_folder).check_returncode()

            # migrate
            cmd = ['python', 'manage.py', 'migrate']
            subprocess.run(cmd, env=self.get_env(), cwd=self.project_folder).check_returncode()

            # create admin
            cmd = ['python', 'manage.py', 'createsuperuser']
            subprocess.run(cmd, env=self.get_env(), cwd=self.project_folder).check_returncode()

        except FileExistsError:
            print('Folder already exists')
            return False

        except subprocess.CalledProcessError as e:
            print('Installation failed: {}'.format(e))
            return False

        return True



import subprocess
from os import path, environ, getcwd


class Runner(object):

    """
    This class manage an existing SIB project
    """

    def __init__(self, project_folder=None):

        """Init"""

        if project_folder:
            self.project_folder = path.abspath(project_folder)
        else:
            self.project_folder = getcwd()

    def get_env(self):

        """Return environment"""

        env = environ.copy()

        # override path if venv is found in project folder
        venv_path = path.join(self.project_folder, 'env', 'bin')
        if path.isdir(venv_path):
            env['PATH'] = venv_path + ":" + env['PATH']

        return env


    def start_project(self):

        """Start the SIB project"""

        try:
            # wrap the django runverver command
            cmd = ['python', 'manage.py', 'runserver']
            subprocess.run(cmd, env=self.get_env(), cwd=self.project_folder)

        except subprocess.CalledProcessError as e:
            print('Start server failed: {}'.format(e))




import click
from . import __version__
from .installer import Installer
from .runner import Runner

# click entrypoint
@click.group()
def main():
    """Startin'Blox installer"""
    pass

@main.command()
@click.argument('name', nargs=1)
@click.argument('directory', required=False)
@click.option('--venv', is_flag=True, default=False, help='Create a virtualenv')
@click.option('-m', '--modules', multiple=True, help='List of modules to install')
def startproject(name, venv, directory=None, modules=None):

    """Start a new startin'blox project"""

    installer = Installer(
        project_name=name,
        project_folder=directory,
        venv=venv,
        modules=list(modules)
    )
    installer.create_project()

@main.command()
def runserver():

    """Start a local server"""

    runner = Runner()
    runner.start_project()

@main.command()
def version():
    """Print module version"""
    click.echo(__version__)



import pytest
import click
from click.testing import CliRunner
from os import path
from sib.cli import main as cli

class TestCli(object):

    """Test the process of project creation"""

    def test_startproject_by_folder(self, tmp_path):

        runner = CliRunner()
        result = runner.invoke(cli, ['startproject', 'sibtest', path.join(tmp_path, 'sibtest')])
        assert result.exit_code == 0
